package sqlite;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;

public class QueryResult {
	public static void print(ResultSet rs, String[][] columnInfo) {
		System.out.println();
    	ArrayList<Object[]> tuples = new ArrayList<Object[]>();
    	int numberOfColumns = 0;
    	try{
    	    ResultSetMetaData metaData = rs.getMetaData();
    	    numberOfColumns = metaData.getColumnCount();
    		   	    
	        int row = 0;
	        while (rs.next()) {
        		tuples.add(new Object[numberOfColumns]);
	        	for(int col = 1; col <= numberOfColumns; col++) {
	        		Object tupleValue = getTupleValue(rs, col, columnInfo[col-1][1]);
	        		tuples.get(row)[col-1] = tupleValue;
	        	}
	        	row++;
            }
	    }catch(SQLException sqlException) {
	        System.err.println("Could NOT get information from result set");
	    }
	    
	    int numberOfRows = tuples.size();

	    String[][] stringValues = getStringValues(numberOfRows, numberOfColumns, tuples, columnInfo);
	    int[] maxWidths = getMaxWidths(tuples.size(), numberOfColumns, columnInfo, stringValues);
	    
	    
	    for(int col = 0; col < numberOfColumns; col++) {
	    	System.out.print(padLeft(columnInfo[col][0], maxWidths[col]) + " ");
	    }
	    System.out.println();
	    for(int col = 0; col < numberOfColumns; col++) {
	    	for(int i = 0; i < maxWidths[col]; i++) {
	    		System.out.print("-");
	    	}
	    	System.out.print(" ");
	    }
	    System.out.println();
	    
	    for(int row = 0; row < numberOfRows; row++) {
	    	for(int col = 0; col < numberOfColumns; col++) {
	    		System.out.print(padLeft(stringValues[row][col], maxWidths[col]) + " ");
	    	}
	    	System.out.println();
	    }
	}

    
    private static String padLeft(String str, int width) {
    	
    	int numberOfSpaces = 0;
    	if(str == null) {
    		numberOfSpaces = width - 4;
    	} else {
    		numberOfSpaces = width - str.length();
    	}
    	
    	StringBuilder result = new StringBuilder();
    	for(int i = 0; i < numberOfSpaces; i++) {
    		result.append(' ');
    	}
    	
    	if(str == null) {
    		result.append("NULL");
    	} else {
    		result.append(str);
    	}
    	
    	return result.toString();
    }
    
    private static int[] getMaxWidths(int numberOfRows, int numberOfColumns, String[][] columnInfo, String[][] stringValues) {
    	int[] result = new int[columnInfo.length];
    	for(int col = 0; col < numberOfColumns; col++) {
    		result[col] = columnInfo[col][0].length();
    	}
    	
    	for(int row = 0; row < numberOfRows; row++) {
    		for(int col = 0; col < numberOfColumns; col++) {
    			int width = 0;
    			if(stringValues[row][col] == null) {
    				width = 4;
    			} else {
    			    width = stringValues[row][col].length();
    			}
    			
    			if(result[col] < width) {
    				result[col] = width;
    			}
    		}
    	}
    
    	return result;
    }
    
    private static String[][] getStringValues(int rows, int cols, ArrayList<Object[]> tuples, String[][] columnInfo) {
    	String[][] result = new String[rows][cols];
    	for(int row = 0; row < rows; row++) {
    		for(int col = 0; col < cols; col++) {
    			switch(columnInfo[col][1]) {
    	       		case "INTEGER":
            		    result[row][col] = Integer.toString((Integer)tuples.get(row)[col]);
        			    break;
        		    case "TEXT":
            		    result[row][col] = (String)tuples.get(row)[col];
        			    break;
        		    case "REAL":
            		    result[row][col] = Double.toString((Double)tuples.get(row)[col]);
        			    break;
        		    case "NULL":
            		    result[row][col] = "NULL";
        			    break;
        		    case "BLOB":
            		    result[row][col] = "UNKNOWN";
        			    break;
        	    }    			
    		}
    	}
    	return result;
    	
    }
    
    private static Object getTupleValue(ResultSet rs, int column, String type) {
    	Object result = null;
    	try{
    		switch(type) {
    			case "INTEGER":
    				result = (Object)rs.getInt(column);
    				break;
    			case "TEXT":
    				result = (Object)rs.getString(column);
    				break;
    			case "REAL":
    				result = (Object)rs.getDouble(column);
    				break;
    			case "NULL":
    				break;
    			case "BLOB":
    				result = null;
    				break;
    		}
    	}catch(SQLException e) {
    		System.err.println("ERROR which trying to get value of type " + type + " from column " + column);
    	}
    	return result;
    }
}

