package sqlite;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;

public class SetUp4 {
	public static Connection initialize() throws Exception {
        Class.forName("org.sqlite.JDBC");
        Connection connection = DriverManager.getConnection("jdbc:sqlite:sample3.db");
		initializePerson(connection);
		initializeChildren(connection);
		dropPersonsChildrenView(connection);
		return connection;
	}

    private static void dropPersonsChildrenView(Connection connection) {
        try{
            Statement stat = connection.createStatement();
            stat.executeUpdate("drop view if exists PersonsChildren;");
        }catch(SQLException sqlException) {
            System.err.println("Could NOT drop PersonsChildren");
        }
    }

    private static void initializePerson(Connection connection) {
        try{
            Statement stat = connection.createStatement();
            stat.executeUpdate("drop table if exists Person;");
            stat.executeUpdate("create table Person (PersonId INTEGER PRIMARY KEY, PersonName TEXT);");
            PreparedStatement prep = connection.prepareStatement( "insert into Person values (?, ?);");
            prep.setString(1, "1");
            prep.setString(2, "Peter Gabriel");
            prep.addBatch();
            prep.setString(1, "2");
            prep.setString(2, "Pearl Hornsby");
            prep.addBatch();
            prep.setString(1, "3");
            prep.setString(2, "Lyle Lovett");
            prep.addBatch();
            prep.setString(1, "4");
            prep.setString(2, "Alice Wilson");
            prep.addBatch();
            prep.setString(1, "5");
            prep.setString(2, "Ed Ek");
            prep.addBatch();

            connection.setAutoCommit(false);
            prep.executeBatch();
            connection.setAutoCommit(true);
        }catch(SQLException sqlException) {
            System.err.println("Could NOT initialize Person");
            System.err.println(sqlException);
        }
    }

    private static void initializeChildren(Connection connection) {
        try{
            Statement stat = connection.createStatement();
            stat.executeUpdate("drop table if exists Children;");
            stat.executeUpdate("create table Children (ParentId INTEGER REFERENCES Person(PersonId), ChildId INTEGER REFERENCES Person(PersonId));");
            stat.execute("PRAGMA foreign_keys=ON;");
            PreparedStatement prep = connection.prepareStatement( "insert into Children values (?, ?);");
            prep.setString(1, "1");
            prep.setString(2, "2");
            prep.addBatch();
            prep.setString(1, "1");
            prep.setString(2, "3");
            prep.addBatch();
            prep.setString(1, "4");
            prep.setString(2, "2");
            prep.addBatch();
            prep.setString(1, "4");
            prep.setString(2, "3");
            prep.addBatch();
            prep.setString(1, "5");
            prep.setString(2, "4");

            connection.setAutoCommit(false);
            prep.executeBatch();
            connection.setAutoCommit(true);
        }catch(SQLException sqlException) {
            System.err.println("Could NOT initialize Children");
            System.err.println(sqlException);
        }
    }
}

